/*
 * Copyright (c) 1993-1994, by Cray Research Superservers, Inc.
 */

#ifndef _SYS_PARTN_H
#define	_SYS_PARTN_H

#pragma ident	"@(#)partn.h	1.6	95/07/28 CRS"

#ifdef	__cplusplus
extern "C" {
#endif

#include <sys/disp.h>
#include <sys/types.h>

#define N_PARTN	  64
#define SYS_PARTN 0

typedef u_longlong_t            cpumask_t;
typedef u_longlong_t            ptnmask_t;
typedef short                   partnid_t;
typedef u_short                 partnattr_t;


/*
 * Partition structure for partitioning processors into disjoint
 * sets. The partition table of 64 struct partition entries are
 * defined statically in disp.c
 */

typedef struct partition {
        partnid_t	partn_id;	/* partition id */
        partnattr_t	partn_attrs;	/* partition attributes */
        cpumask_t	partn_mask;	/* partition bit mask for cpus 0-63 */
#ifdef _CRS_RMSCER
	int		partn_rss;	/* The RSS for this partn */
	int		partn_quota;	/* The RSS working set for partn */
	kmutex_t	partn_rssmutex;	/* mutex to controll rss field */
#endif /* _CRS_RMSCER */
} partn_t;

#ifdef _CRS_RMSCER
extern u_int crs_partn_rm;

/*
 * This is a shadow struct for the hat structures. partn_data contains
 * fields that we would like to track on a per hat basis. There is 
 * exactly on partn_data struct for every hat struct in the system.
 * They are tracked as arrays of structs, partns[nhats] and hats[nhats].
 */
typedef struct partn_data {
	partnid_t	p_partnid;	/* partition id of this hat */
	int		p_urss;		/* the unique RSS of this hat */
	struct hat	*p_hat;		/* for debugging track hat ptr */
} partnd_t;
extern partnd_t *partns;

#define t_to_partn(t)	(partns[ttoproc((t))->p_as->a_hat - hats])
#define PARTN_URSS(t)		(t_to_partn(t).p_urss)
#define PARTN_ID(t)		(t_to_partn(t).p_partnid)
#endif /* _CRS_RMSCER */
/* 
 * Partition attributes
 */
#define	PARTN_ATTR_CAN_BORROW	0x01	/* can borrow threads to run */
#define	PARTN_ATTR_CAN_LOAN	0x02	/* can loan out threads to run  */
#define	PARTN_ATTR_NO_SBUS_INTS	0x04	/* disable sbus interrupts */
#define PARTN_ATTR_SYSTEM	0x80	/* system partition */
#define PARTN_ATTR_VALID	(PARTN_ATTR_CAN_BORROW | \
				 PARTN_ATTR_CAN_LOAN | \
                                 PARTN_ATTR_NO_SBUS_INTS)

#define PARTN_ATTR_VALID_SYS	(PARTN_ATTR_CAN_BORROW | \
				 PARTN_ATTR_CAN_LOAN | \
				 PARTN_ATTR_SYSTEM)
				
/* flag values for PTN_ASSIGN_CPU */

#define PARTN_ASSIGN_FORCE	0x01	/* force the assignment of the CPUs
					   even if they have bound threads.*/

/* Macro for determining whether a cpu is in the partition. */
#define CPU_IN_PARTITION(partn_mask,cpu_num) \
	((partn_mask) & ((u_longlong_t) 1 << (cpu_num)))

#ifdef	__cplusplus
}
#endif

#endif /* _SYS_PARTN_H */
